@@ -2,6 +2,15 @@
 
 set -eux
 
+function copy_logs() {
+  log_dir=/host-rw/logs
+  mkdir -p ${log_dir}
+  cp -a /var/log/postgresql ${log_dir} || :
+  mkdir -p ${log_dir}/pgroonga/ || :
+  cp -a ${data_dir}/pgroonga*.log* ${log_dir}/pgroonga/ || :
+  chmod -R go+rx ${log_dir}
+}
+
 function run_test() {
   echo "::group::Run test"
 
@@ -20,23 +29,28 @@ function run_test() {
     echo "::group::Diff"
     cat regression.diffs
     echo "::endgroup::"
-    mkdir -p /host-rw/logs
+    copy_logs
     cp -a regression.diffs /host-rw/logs/
-    cp -a ${data_dir}/log /host-rw/logs/postgresql || :
-    mkdir -p /host-rw/logs/pgroonga/ || :
-    cp -a ${data_dir}/pgroonga.log* /host-rw/logs/pgroonga/ || :
-    chmod -R go+rx /host-rw/logs/
     exit ${pg_regress_status}
   fi
 
 
   echo "::group::Test primary-maintainer"
 
+  set +e
   $(pg_config --bindir)/createuser postgres --superuser
   /host/packages/test-primary-maintainer.sh
+  test_status=$?
   $(pg_config --bindir)/dropuser postgres
+  set -e
 
   echo "::endgroup::"
+
+
+  if [ ${test_status} -ne 0 ]; then
+    copy_logs
+    exit ${test_status}
+  fi
 }
 
 echo "::group::Prepare APT repositories"
@@ -73,10 +87,14 @@ if [ "${os}" = "ubuntu" ]; then
 fi
 
 if find ${repositories_dir} | grep -q "pgdg"; then
-  echo "deb http://apt.postgresql.org/pub/repos/apt/ ${code_name}-pgdg main" | \
-    tee /etc/apt/sources.list.d/pgdg.list
-  wget --quiet -O - https://www.postgresql.org/media/keys/ACCC4CF8.asc | \
-    apt-key add -
+  wget -O /usr/share/keyrings/pgdg.asc \
+    https://www.postgresql.org/media/keys/ACCC4CF8.asc
+  (echo "Types: deb"; \
+   echo "URIs: http://apt.postgresql.org/pub/repos/apt"; \
+   echo "Suites: ${code_name}-pgdg"; \
+   echo "Components: main"; \
+   echo "Signed-By: /usr/share/keyrings/pgdg.asc") | \
+    tee /etc/apt/sources.list.d/pgdg.sources
 fi
 
 echo "::endgroup::"
